library tree_view;

import 'dart:html';


class TreeView {

  String indent_width = '1.5em';
  Element element = new DivElement();

  // Inject the CSS used by this class, if not already injected.
  void injectCss() {
    if (querySelector('[href="/assets/web_widgets/tree_view.css"]') == null) {
      Element linkElement = new LinkElement()
      ..rel = 'stylesheet'
      ..href = '/assets/web_widgets/tree_view.css';
      querySelector('body').children.add(linkElement);
    }
  }

  // Construct a TreeView from a list, populates this.element.
  TreeView.fromList(List<Map> treeViewList) {
    injectCss();
    this.element.classes.add('tree-view');
    for (var item in treeViewList) {
      this.element.children.add(_treeViewNodeMapToDom(item));
    }
  }

  void openNode(String nodeId) {
    querySelector('#${nodeId} .tree-view-node-name').click();
  }

  // Return Elemente (used as a tree view node) from a Map.
  Element _treeViewNodeMapToDom(var map, [int indent_level=0]) {
    Element node;
    if (map.containsKey('anchor')) {
      node = new AnchorElement()
      ..href = map['anchor'];
    }
    else {
      node = new DivElement();
    }
    node.classes.add('tree-view-node');
    node.id = map['id'];

    DivElement nodeName = new DivElement()
    ..classes.add('tree-view-node-name');

    // Add indent to .tree-view-node-name 
    for (int counter=0; counter<=indent_level; counter++){
      DivElement indenter = new DivElement()
      ..style.display  = 'inline-block'
      ..style.width    = "${this.indent_width}"
      ..style.minWidth = "${this.indent_width}"
      ..style.maxWidth = "${this.indent_width}";
      nodeName.children.add(indenter);
    }

    if (map.containsKey('icon')) {
      // Use Font Awesome as icon 
      nodeName.children.add(new Element.html('''<i class="fa ${map['icon']} fa-fw"></i>'''));
    }

    nodeName.children.add(new Element.html('''<span>${map['name']}</span>'''));
    node.children.add(nodeName);

    nodeName.onClick.listen(
      (_) {
        for (var elem in querySelectorAll('.tree-view-node-name')) {
          elem.classes.remove('active-tree-view-node-name');
        }
        nodeName.classes.add('active-tree-view-node-name');
      }
    );


    // Only responde for click on first level .tree-view-node-name
    if (indent_level == 0) {
      // TODO Write a better selector
      // This will select all .tree-view-node-child in the current page,
      // other than the ones inside the current .tree-view.
      nodeName.onClick.listen((_) => querySelectorAll('.tree-view-node-child').style.display = 'none');
    }

    if (map.containsKey('onClick')) {
      // TODO Function argument to be fixed
      nodeName.onClick.listen((_) => map['onClick'][0](map['onClick'][1]));
    }

    if (map.containsKey('child')) {
      DivElement nodeChild = new DivElement()
      ..classes.add('tree-view-node-child');
      for (var item in map['child']) {
        nodeChild.children.add(_treeViewNodeMapToDom(item, indent_level+1));
      }
      node.children.add(nodeChild);

      // Show the child nodes when clicked 
      nodeName.onClick.listen((_) => querySelector('#${map["id"]}>.tree-view-node-child').style.display = 'block');
    }

    return node;
  }

}
